from django.contrib.auth.models import Group, Permission
from django.db import transaction
from shared_schema_tenants.settings import DEFAULT_TENANT_OWNER_PERMISSIONS


def get_current_tenant():
    from shared_schema_tenants.middleware import TenantMiddleware
    return TenantMiddleware.get_current_tenant()


def set_current_tenant(tenant_slug):
    from shared_schema_tenants.middleware import TenantMiddleware
    return TenantMiddleware.set_tenant(tenant_slug)


def clear_current_tenant():
    from shared_schema_tenants.middleware import TenantMiddleware
    return TenantMiddleware.clear_tenant()


def create_default_tenant_groups():
    with transaction.atomic():
        group, created = Group.objects.get_or_create(name='tenant_owner')

        if created:
            for perm in DEFAULT_TENANT_OWNER_PERMISSIONS:
                try:
                    group.permissions.add(Permission.objects.get(
                        content_type__app_label=perm.split('.')[0],
                        codename=perm.split('.')[1]))
                except Permission.DoesNotExist:
                    pass

        return [group]



from rest_framework import permissions
from shared_schema_tenants.helpers.tenants import get_current_tenant


class DjangoTenantModelPermission(permissions.DjangoModelPermissions):

    perms_map = {
        'GET': ['%(app_label)s.view_%(model_name)s'],
        'OPTIONS': [],
        'HEAD': [],
        'POST': ['%(app_label)s.add_%(model_name)s'],
        'PUT': ['%(app_label)s.change_%(model_name)s'],
        'PATCH': ['%(app_label)s.change_%(model_name)s'],
        'DELETE': ['%(app_label)s.delete_%(model_name)s'],
    }

    def has_permission(self, request, view):
        tenant = get_current_tenant()
        return (super(DjangoTenantModelPermission, self).has_permission(request, view) and
                tenant and
                request.user.relationships.filter(tenant=tenant).exists())

    def has_object_permission(self, request, view, obj):
        return request.user.relationships.filter(tenant=obj.tenant).exists()





class TenantNotFoundError(Exception):

    def __init__(self, message="Tenant with this slug couldn't be found", errors=[]):
        super(TenantNotFoundError, self).__init__(message)


class TenantFieldTypeConfigurationError(Exception):

    def __init__(self, message="The field is not configured correctly", errors=[]):
        super(TenantFieldTypeConfigurationError, self).__init__(message)



from django.conf import settings

tenant_settings = getattr(settings, 'SHARED_SCHEMA_TENANTS', {})


TENANT_SERIALIZER = (tenant_settings
                     .get('SERIALIZERS', {})
                     .get('TENANT_SERIALIZER',
                          'shared_schema_tenants.serializers.TenantSerializer'))


TENANT_SITE_SERIALIZER = (tenant_settings
                          .get('SERIALIZERS', {})
                          .get('TENANT_SITE_SERIALIZER',
                               'shared_schema_tenants.serializers.TenantSiteSerializer'))

TENANT_SETTINGS_SERIALIZER = (tenant_settings
                              .get('SERIALIZERS', {})
                              .get('TENANT_SETTINGS_SERIALIZER',
                                   'shared_schema_tenants.serializers.TenantSettingsSerializer'))

TENANT_RELATIONSHIP_SERIALIZER = (
    tenant_settings
    .get('SERIALIZERS', {})
    .get('TENANT_SITE_SERIALIZER',
         'shared_schema_tenants.serializers.TenantSiteSerializer'))


DEFAULT_TENANT_SETTINGS_FIELDS = tenant_settings.get(
    'DEFAULT_TENANT_SETTINGS_FIELDS', {})


DEFAULT_TENANT_SETTINGS = {key: value.get('default')
                           for key, value
                           in DEFAULT_TENANT_SETTINGS_FIELDS.items()}


DEFAULT_TENANT_EXTRA_DATA_FIELDS = tenant_settings.get(
    'DEFAULT_TENANT_EXTRA_DATA_FIELDS',
    {})

DEFAULT_TENANT_EXTRA_DATA = {key: value.get('default')
                             for key, value
                             in DEFAULT_TENANT_EXTRA_DATA_FIELDS.items()}

DEFAULT_SITE_DOMAIN = tenant_settings.get(
    'DEFAULT_SITE_DOMAIN',
    'localhost'
)

DEFAULT_TENANT_SLUG = tenant_settings.get(
    'DEFAULT_TENANT_SLUG',
    'default'
)

TENANT_HTTP_HEADER = tenant_settings.get('TENANT_HTTP_HEADER', 'Tenant-Slug')


DEFAULT_TENANT_OWNER_PERMISSIONS = tenant_settings.get(
    'DEFAULT_TENANT_OWNER_PERMISSIONS', [
        'shared_schema_tenants.add_tenantsite',
        'shared_schema_tenants.change_tenantsite',
        'shared_schema_tenants.delete_tenantsite',
        'shared_schema_tenants.add_tenantrelationship',
        'shared_schema_tenants.delete_tenantrelationship',
        'shared_schema_tenants.change_tenantrelationship',
    ])



from django.db import transaction
from django.utils.text import ugettext_lazy as _
from django.contrib.sites.models import Site
from django.core.exceptions import ValidationError

from rest_framework import serializers

from shared_schema_tenants.models import (Tenant, TenantSite)
from shared_schema_tenants.helpers.tenants import (
    create_default_tenant_groups, get_current_tenant)
from shared_schema_tenants.helpers import (
    TenantSettingsHelper, TenantExtraDataHelper)


class TenantSerializer(serializers.ModelSerializer):
    extra_data = serializers.JSONField()

    class Meta:
        model = Tenant
        fields = ['name', 'slug', 'extra_data',]

    def validate_extra_data(self, extra_data):
        extra_data_helper = TenantExtraDataHelper()
        try:
            validated_extra_data = extra_data_helper.validate_fields(
                self.context, extra_data, partial=self.partial)
        except ValidationError as e:
            raise serializers.ValidationError(e.message_dict)
        else:
            return validated_extra_data

    def create(self, validated_data):
        with transaction.atomic():
            tenant = Tenant.objects.create(**validated_data)

            rel = tenant.relationships.create(user=self.context['request'].user)
            rel.groups.add(create_default_tenant_groups()[0])

            return tenant

    def update(self, instance, validated_data):
        instance.name = validated_data.get('name', instance.name)
        instance.slug = validated_data.get('slug', instance.slug)

        extra_data_helper = TenantExtraDataHelper(instance=instance)
        instance = extra_data_helper.update_fields(
            validated_data.get('extra_data', {}), commit=False)

        instance.save()

        return instance


class TenantSettingsSerializer(serializers.Serializer):

    def is_valid(self, raise_exception=False):
        settings_helper = TenantSettingsHelper()
        try:
            self._validated_data = settings_helper.validate_fields(
                self.context, self.initial_data, self.partial)
        except ValidationError as exc:
            self._validated_data = {}
            self._errors = exc.detail
        else:
            self._errors = {}

        if self._errors and raise_exception:
            raise ValidationError(self.errors)

        return not bool(self._errors)

    def create(self, validated_data):
        settings_helper = TenantSettingsHelper()
        instance = settings_helper.update_fields(self.validated_data, partial=self.partial)
        return instance

    def to_representation(self, obj):
        return obj.settings


class TenantSiteSerializer(serializers.Serializer):
    id = serializers.IntegerField(read_only=True)
    domain = serializers.CharField(required=True)

    def to_representation(self, instance):
        if instance:
            return {'id': instance.id, 'domain': instance.site.domain}
        return None

    def validate_domain(self, domain):
        if Site.objects.filter(domain=domain).exists():
            raise ValidationError(_((
                'This domain is already being used '
                'by another organization')))

        return domain

    def create(self, validated_data):
        tenant = get_current_tenant()
        domain = validated_data['domain']

        with transaction.atomic():
            site = Site.objects.create(name=domain, domain=domain)
            return TenantSite.objects.create(tenant=tenant, site=site)




from django.db import models
from django.conf import settings as django_settings
from django.contrib.sites.models import Site
from django.db.models.signals import post_delete


import json

from model_utils.models import TimeStampedModel

from shared_schema_tenants.managers import (
    SingleTenantModelManager)
from shared_schema_tenants.settings import DEFAULT_TENANT_SETTINGS, DEFAULT_TENANT_EXTRA_DATA
from shared_schema_tenants.validators import validate_json


class Tenant(TimeStampedModel):
    name = models.CharField(max_length=255)
    slug = models.CharField(max_length=255, primary_key=True)

    if 'postgresql' in django_settings.DATABASES['default']['ENGINE']:
        from django.contrib.postgres.fields import JSONField
        extra_data = JSONField(blank=True, null=True,
                               default=DEFAULT_TENANT_EXTRA_DATA)
        settings = JSONField(blank=True, null=True,
                             default=DEFAULT_TENANT_SETTINGS)
    else:
        _extra_data = models.TextField(blank=True, null=True,
                                       validators=[validate_json],
                                       default=json.dumps(DEFAULT_TENANT_EXTRA_DATA))
        _settings = models.TextField(blank=True, null=True,
                                     validators=[validate_json],
                                     default=json.dumps(DEFAULT_TENANT_SETTINGS))

        @property
        def extra_data(self):
            import json
            return json.loads(self._extra_data)

        @extra_data.setter
        def extra_data(self, value):
            import json
            self._extra_data = json.dumps(value)

        @property
        def settings(self):
            import json
            return json.loads(self._settings)

        @settings.setter
        def settings(self, value):
            import json
            self._settings = json.dumps(value)

    def __str__(self):
        return self.name


class TenantSite(TimeStampedModel):
    tenant = models.ForeignKey('Tenant', related_name="tenant_sites")
    site = models.OneToOneField(Site, related_name="tenant_site")

    objects = SingleTenantModelManager

    def __str__(self):
        return '%s - %s' % (self.tenant.name, self.site.domain)


def post_delete_tenant_site(sender, instance, *args, **kwargs):
    if instance.site:
        instance.site.delete()
post_delete.connect(post_delete_tenant_site, sender=TenantSite)


class TenantRelationship(TimeStampedModel):
    tenant = models.ForeignKey('Tenant', related_name="relationships")
    user = models.ForeignKey(django_settings.AUTH_USER_MODEL, related_name="relationships")
    groups = models.ManyToManyField('auth.Group',
                                    related_name="user_tenant_groups",
                                    blank=True)
    permissions = models.ManyToManyField('auth.Permission',
                                         related_name="user_tenant_permissions",
                                         blank=True)

    def __str__(self):
        groups_str = ', '.join([g.name for g in self.groups.all()])
        return '%s - %s (%s)' % (str(self.user), str(self.tenant), groups_str)

    class Meta:
        unique_together = [['user', 'tenant']]



from django.core.exceptions import ValidationError
from django.utils.text import ugettext_lazy as _
from shared_schema_tenants.exceptions import TenantFieldTypeConfigurationError
from shared_schema_tenants.helpers.tenants import get_current_tenant


class TenantJSONFieldHelper(object):
    TYPES_TO_INTERNAL_MAP = {
        'string': [str],
        'number': [int, float],
        'boolean': [bool],
        'list': [list],
        'object': [dict],
    }

    def __init__(self, instance_field_name, instance=None,
                 tenant_fields={}, tenant_default_fields_values={}):
        if not instance:
            instance = get_current_tenant()

        self.instance = instance
        self.instance_field_name = instance_field_name
        self.tenant = get_current_tenant()
        self.tenant_fields = tenant_fields
        self.tenant_default_fields_values = tenant_default_fields_values

    def get_tenant_fields(self):
        return getattr(self, 'tenant_fields', {})

    def get_tenant_default_fields_values(self):
        return getattr(self, 'tenant_default_fields_values', {})

    def get_tenant(self):
        return self.instance

    def get_field(self, instance, field_key):
        fields = getattr(instance, self.instance_field_name)
        if fields:
            return fields.get(field_key, None)

        return None

    def validate_field(self, context, key, value, original_value=None):
        tenant_fields = self.get_tenant_fields()

        try:
            field = tenant_fields[key]
        except KeyError:
            raise ValidationError(_('%(field)s is not a valid field') % {'field': key})

        try:
            field_type = field['type']
        except KeyError:
            raise TenantFieldTypeConfigurationError((
                'You must define a valid type for tenant setting '
                'field "%(field)s" in "TENANT_SETTINGS_FIELDS"') % {'field': key})

        if field_type not in self.TYPES_TO_INTERNAL_MAP.keys():
            raise TenantFieldTypeConfigurationError((
                '"%(wrong_type)s" of field %(field)s is not a valid type. '
                'Use one of these: %(valid_types)s') % {
                    'wrong_type': field_type,
                    'field': key,
                    'valid_types': ', '.join(
                        [('"%s"' % t) for t in self.TYPES_TO_INTERNAL_MAP.keys()])
            })

        if tenant_fields[key].get('required', True):
            if value == None or value == '':  # noqa
                raise ValidationError({
                    key: [
                        _('This field is required')
                    ]
                })

            if type(value) not in self.TYPES_TO_INTERNAL_MAP[field_type]:
                raise ValidationError({
                    key: [
                        _('%(field)s must be a valid %(field_type)s') % {
                            'field': key, 'field_type': field_type}
                    ]
                })

        for validator in field.get('validators', []):
            try:
                value = validator(context, value, original_value)
            except ValidationError as e:
                raise ValidationError({key: [e]})
        return value

    def validate_fields(self, context, data, partial=False):
        errors = {}
        has_errors = False
        for key in self.get_tenant_fields().keys():
            value = data.get(key)
            if not value and partial:
                continue

            try:
                data[key] = self.validate_field(
                    context, key, value, self.get_field(self.tenant, key))
            except ValidationError as e:
                has_errors = True
                errors = dict(errors, **e.message_dict)

        if has_errors:
            raise ValidationError(errors)

        return data

    def update_fields(self, validated_data, partial=False, commit=True):
        if not partial:
            setattr(
                self.instance,
                self.instance_field_name,
                validated_data
            )
        else:
            setattr(
                self.instance,
                self.instance_field_name,
                dict(self.tenant.settings, **validated_data)
            )

        if commit:
            self.instance.save()

        return self.instance

    def update_field(self, key, value, commit=True):
        return self.update_fields({key: value}, partial=True, commit=commit)



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals

from django.conf.urls import url

from . import views

urlpatterns = [
    url(
        regex=r'^organization/$',
        view=views.TenantListView.as_view(),
        name='tenant_list'
    ),
    url(
        regex=r'^organization/(?P<pk>[\w.@+-]+)/$',
        view=views.TenantDetailsView.as_view(),
        name='tenant_details'
    ),
    url(
        regex=r'^organization-site/$',
        view=views.TenantSiteListView.as_view(),
        name='tenant_site_list'
    ),
    url(
        regex=r'^organization-site/(?P<pk>[\d]+)/$',
        view=views.TenantSiteDetailsView.as_view(),
        name='tenant_site_details'
    ),
]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.4 on 2017-08-13 23:37
from __future__ import unicode_literals

import django.utils.version
from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone
import model_utils.fields
import shared_schema_tenants.validators

try:
    import django.contrib.postgres.fields.jsonb
except ImportError:
    pass


sites_dep_migration = ('sites', '0002_alter_domain_unique',)
if django.utils.version.get_complete_version()[1] < 10:
    sites_dep_migration = ('sites', '0001_initial',)


tenant_fields = [
    ('created', model_utils.fields.AutoCreatedField(
        default=django.utils.timezone.now, editable=False, verbose_name='created')),
    ('modified', model_utils.fields.AutoLastModifiedField(
        default=django.utils.timezone.now, editable=False, verbose_name='modified')),
    ('name', models.CharField(max_length=255)),
    ('slug', models.CharField(max_length=255, primary_key=True, serialize=False)),
]

if 'postgresql' in settings.DATABASES['default']['ENGINE']:
    tenant_fields.extend([
        ('settings', django.contrib.postgres.fields.jsonb.JSONField(
            blank=True, default={}, null=True)),
        ('extra_data', django.contrib.postgres.fields.jsonb.JSONField(
            blank=True, default={}, null=True)),
    ])
else:
    tenant_fields.extend([
        ('_settings', models.TextField(blank=True, default="", null=True,
                                       validators=[shared_schema_tenants.validators.validate_json])),
        ('_extra_data', models.TextField(blank=True, default={}, null=True,
                                         validators=[shared_schema_tenants.validators.validate_json])),
    ])


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        sites_dep_migration,
    ]

    operations = [
        migrations.CreateModel(
            name='Tenant',
            fields=tenant_fields,
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='TenantRelationship',
            fields=[
                ('id', models.AutoField(auto_created=True,
                                        primary_key=True, serialize=False, verbose_name='ID')),
                ('created', model_utils.fields.AutoCreatedField(
                    default=django.utils.timezone.now, editable=False, verbose_name='created')),
                ('modified', model_utils.fields.AutoLastModifiedField(
                    default=django.utils.timezone.now, editable=False, verbose_name='modified')),
                ('groups', models.ManyToManyField(blank=True,
                                                  related_name='user_tenant_groups', to='auth.Group')),
                ('permissions', models.ManyToManyField(blank=True,
                                                       related_name='user_tenant_permissions', to='auth.Permission')),
                ('tenant', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE,
                                             related_name='relationships', to='shared_schema_tenants.Tenant')),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE,
                                           related_name='relationships', to=settings.AUTH_USER_MODEL)),
            ],
        ),
        migrations.CreateModel(
            name='TenantSite',
            fields=[
                ('id', models.AutoField(auto_created=True,
                                        primary_key=True, serialize=False, verbose_name='ID')),
                ('created', model_utils.fields.AutoCreatedField(
                    default=django.utils.timezone.now, editable=False, verbose_name='created')),
                ('modified', model_utils.fields.AutoLastModifiedField(
                    default=django.utils.timezone.now, editable=False, verbose_name='modified')),
                ('site', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE,
                                              related_name='tenant_site', to='sites.Site')),
                ('tenant', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE,
                                             related_name='tenant_sites', to='shared_schema_tenants.Tenant')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.AlterUniqueTogether(
            name='tenantrelationship',
            unique_together=set([('user', 'tenant')]),
        ),
    ]



from shared_schema_tenants.settings import (
    DEFAULT_TENANT_SETTINGS_FIELDS, DEFAULT_TENANT_SETTINGS)

from shared_schema_tenants.helpers.tenant_json_field import TenantJSONFieldHelper


class TenantSettingsHelper(TenantJSONFieldHelper):

    def __init__(self, instance=None):
        super(TenantSettingsHelper, self).__init__(
            instance_field_name='settings', instance=instance,
            tenant_fields=DEFAULT_TENANT_SETTINGS_FIELDS,
            tenant_default_fields_values=DEFAULT_TENANT_SETTINGS)



import logging
from django.contrib.auth.models import Permission
from django.contrib.auth.backends import ModelBackend
from django.contrib.auth import get_user_model
from django.db.models import Q

from shared_schema_tenants.models import TenantRelationship
from shared_schema_tenants.helpers.tenants import get_current_tenant


UserModel = get_user_model()

logger = logging.getLogger(__name__)


class TenantModelBackend(ModelBackend):
    """
    Authenticates against settings.AUTH_USER_MODEL.
    """

    def _get_user_global_permissions(self, user_obj):
        return user_obj.user_permissions.all()

    def _get_group_global_permissions(self, user_obj):
        user_groups_field = get_user_model()._meta.get_field('groups')
        user_groups_query = 'group__%s' % user_groups_field.related_query_name()
        return Permission.objects.filter(**{user_groups_query: user_obj})

    def _get_user_tenant_permissions(self, relationship):
        return relationship.permissions.all()

    def _get_group_tenant_permissions(self, relationship):
        relationship_groups_field = TenantRelationship._meta.get_field('groups')
        relationship_groups_query = 'group__%s' % relationship_groups_field.related_query_name()
        return Permission.objects.filter(**{relationship_groups_query: relationship})

    def _get_user_permissions(self, relationship):
        relationship_permissions_field = TenantRelationship._meta.get_field('permissions')
        relationship_permission_query = relationship_permissions_field.related_query_name()

        user_permissions_field = UserModel._meta.get_field('user_permissions')
        user_permission_query = user_permissions_field.related_query_name()

        user_groups_field = get_user_model()._meta.get_field('groups')
        user_groups_query = 'group__%s' % user_groups_field.related_query_name()
        return Permission.objects.filter(
            Q(**{relationship_permission_query: relationship}) |
            Q(**{user_permission_query: relationship.user}) |
            Q(**{user_groups_query: relationship.user})).distinct()

    def _get_group_permissions(self, relationship):
        relationship_groups_field = TenantRelationship._meta.get_field(
            'groups')
        relationship_groups_query = 'group__%s' % relationship_groups_field.related_query_name()
        user_groups_field = get_user_model()._meta.get_field('groups')
        user_groups_query = 'group__%s' % user_groups_field.related_query_name()
        return Permission.objects.filter(
            Q(**{relationship_groups_query: relationship}) |
            Q(**{user_groups_query: relationship.user}))

    def _get_tenant_permissions(self, user_obj, obj, from_name):
        if not user_obj.is_active or user_obj.is_anonymous() or obj is not None:
            return set()

        tenant = get_current_tenant()
        if not tenant:
            return set()

        perm_cache_name = '_tenant_%s_perm_cache' % from_name

        if (not hasattr(user_obj, perm_cache_name) or
                not getattr(user_obj, perm_cache_name).get(tenant.pk)):
            if user_obj.is_superuser:
                relationship_perms = Permission.objects.all()
                relationship_perms = relationship_perms.values_list(
                    'content_type__app_label', 'codename').order_by()
            else:
                try:
                    relationship = TenantRelationship.objects.get(
                        user=user_obj, tenant=tenant)
                except TenantRelationship.DoesNotExist:
                    relationship_perms = set()
                else:
                    relationship_perms = getattr(self, '_get_%s_tenant_permissions' %
                                                 from_name)(relationship)
                    relationship_perms = relationship_perms.values_list(
                        'content_type__app_label', 'codename').order_by()
            setattr(user_obj, perm_cache_name, {
                tenant.pk: {
                    "%s.%s" % (ct, name) for ct, name in relationship_perms
                }
            })

        return getattr(user_obj, perm_cache_name).get(tenant.pk)

    def _get_global_permissions(self, user_obj, obj, from_name):
        if not user_obj.is_active or user_obj.is_anonymous() or obj is not None:
            return set()

        perm_cache_name = '_%s_perm_cache' % from_name
        if not hasattr(user_obj, perm_cache_name):
            if user_obj.is_superuser:
                perms = Permission.objects.all()
            else:
                perms = getattr(self, '_get_%s_global_permissions' % from_name)(user_obj)
            perms = perms.values_list('content_type__app_label', 'codename').order_by()
            setattr(user_obj, perm_cache_name, set("%s.%s" % (ct, name) for ct, name in perms))
        return getattr(user_obj, perm_cache_name)

    def _get_permissions(self, user_obj, obj, from_name):
        return self._get_global_permissions(user_obj, obj, from_name).union(
            self._get_tenant_permissions(user_obj, obj, from_name))

    def get_user_global_permissions(self, user_obj, obj=None):
        return self._get_global_permissions(user_obj, obj, 'user')

    def get_user_tenant_permissions(self, user_obj, obj=None):
        return self._get_tenant_permissions(user_obj, obj, 'user')

    def get_group_global_permissions(self, user_obj, obj=None):
        return self._get_global_permissions(user_obj, obj, 'group')

    def get_group_tenant_permissions(self, user_obj, obj=None):
        return self._get_tenant_permissions(user_obj, obj, 'group')

    def get_all_global_permissions(self, user_obj, obj=None):
        if not user_obj.is_active or user_obj.is_anonymous() or obj is not None:
            return set()
        if not hasattr(user_obj, '_perm_cache'):
            user_obj._perm_cache = self.get_user_global_permissions(user_obj, obj)
            user_obj._perm_cache.update(self.get_group_global_permissions(user_obj, obj))
        return user_obj._perm_cache

    def get_all_tenant_permissions(self, user_obj, obj=None):
        if not user_obj.is_active or user_obj.is_anonymous() or obj is not None:
            return set()

        tenant = get_current_tenant()
        if not tenant:
            return set()

        if (not hasattr(user_obj, '_tenant_perm_cache') or
                not getattr(user_obj, '_tenant_perm_cache').get(tenant.pk)):
            user_obj._tenant_perm_cache = {tenant.pk: self.get_user_tenant_permissions(user_obj, obj)}
            user_obj._tenant_perm_cache[tenant.pk].update(self.get_group_tenant_permissions(user_obj, obj))
        return user_obj._tenant_perm_cache[tenant.pk]

    def get_all_permissions(self, user_obj, obj=None):
        return self.get_all_global_permissions(user_obj, obj).union(
            self.get_all_tenant_permissions(user_obj, obj))



import json
from django.core.exceptions import ValidationError
from django.utils.text import ugettext_lazy as _


def validate_json(value):
    try:
        value = json.loads(value)
    except json.decoder.JSONDecodeError:
        raise ValidationError(_("This field must be a valid json"))



from django.db import transaction
from shared_schema_tenants.models import TenantRelationship


def create_relationship(tenant, user, groups=[], permissions=[]):
    with transaction.atomic():
        relationship = TenantRelationship.objects.create(user=user, tenant=tenant)
        try:
            relationship.groups.set(groups)
            relationship.permissions.set(permissions)
        except AttributeError:
            # compatibility with old django 1.8 and 1.9
            for group in groups:
                relationship.groups.add(group)
            for perm in permissions:
                relationship.permissions.add(perm)

        return relationship



from rest_framework import generics, views, response, status
from django.db import transaction

from shared_schema_tenants.permissions import DjangoTenantModelPermission
from shared_schema_tenants.models import Tenant, TenantSite
from shared_schema_tenants.utils import import_class
from shared_schema_tenants.settings import (
    TENANT_SERIALIZER, TENANT_SITE_SERIALIZER,
    TENANT_SETTINGS_SERIALIZER)
from shared_schema_tenants.helpers.tenants import get_current_tenant


TenantSerializer = import_class(TENANT_SERIALIZER)
TenantSiteSerializer = import_class(TENANT_SITE_SERIALIZER)
TenantSettingsSerializer = import_class(TENANT_SETTINGS_SERIALIZER)


class TenantListView(generics.ListCreateAPIView):
    serializer_class = TenantSerializer
    permission_classes = [DjangoTenantModelPermission]

    def get_queryset(self):
        if self.request.user.is_authenticated:
            return Tenant.objects.filter(
                relationships__user=self.request.user).distinct()
        else:
            return Tenant.objects.none()


class TenantDetailsView(generics.RetrieveUpdateDestroyAPIView):
    serializer_class = TenantSerializer
    permission_classes = [DjangoTenantModelPermission]

    def get_queryset(self):
        if self.request.user.is_authenticated:
            return Tenant.objects.filter(
                relationships__user=self.request.user).distinct()
        else:
            return Tenant.objects.none()

    def get_object(self):
        return get_current_tenant()


class TenantSettingsDetailsView(views.APIView):
    serializer_class = TenantSettingsSerializer
    permission_classes = [DjangoTenantModelPermission]

    def get(self, request, *args, **kwargs):
        serializer = self.get_serializer_class(
            self.request.tenant,
            context={'request': request, 'view': self})
        return response.Response(serializer.data, status=status.HTTP_200_OK)

    def post(self, request, *args, **kwargs):
        serializer = self.get_serializer_class(
            data=self.request.data,
            context={'request': request, 'view': self})

        if serializer.is_valid():
            return response.Response(serializer.data, status=status.HTTP_200_OK)

        return response.Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)


class TenantSiteListView(generics.ListCreateAPIView):
    serializer_class = TenantSiteSerializer
    permission_classes = [DjangoTenantModelPermission]

    def get_queryset(self):
        return TenantSite.objects.filter().distinct()

    def get_serializer(self, *args, **kwargs):
        data = kwargs.get('data', {})
        data['tenant'] = get_current_tenant()
        kwargs['data'] = data
        return super(TenantSiteListView, self).get_serializer(*args, **kwargs)


class TenantSiteDetailsView(generics.DestroyAPIView):
    serializer_class = TenantSiteSerializer
    permission_classes = [DjangoTenantModelPermission]

    def get_queryset(self):
        return TenantSite.objects.filter().distinct()

    def destroy(self, request, *args, **kwargs):
        tenant_site = self.get_object()
        site = tenant_site.site

        with transaction.atomic():
            response = super(TenantSiteDetailsView, self).destroy(request, *args, **kwargs)
            site.delete()

        return response



from django import forms
from django.contrib import admin
from django.contrib.sites.models import Site
from django.core.exceptions import ValidationError
from django.utils.text import ugettext_lazy as _
import json

from shared_schema_tenants.models import Tenant, TenantSite, TenantRelationship
from shared_schema_tenants.helpers.tenant_extra_data import TenantExtraDataHelper
from shared_schema_tenants.helpers.tenant_settings import TenantSettingsHelper


class TenantSiteForm(forms.ModelForm):
    site = forms.CharField(max_length=255)

    class Meta:
        model = TenantSite
        fields = ['site', 'tenant']

    def __init__(self, *args, **kwargs):
        super(TenantSiteForm, self).__init__(*args, **kwargs)
        if kwargs.get('instance'):
            self.initial['site'] = kwargs.get('instance').site.domain

    def clean_site(self):
        site = self.cleaned_data['site']
        instance = getattr(self, 'instance')
        new_site_instance, created = Site.objects.get_or_create(name=self.data['name'], domain=site)
        try:
            old_site_instance = instance.site
        except Site.DoesNotExist:
            old_site_instance = None

        if not created and old_site_instance and old_site_instance != new_site_instance:
            raise ValidationError(
                _('A site with this domain already asigned to an organization'))

        return new_site_instance

    def save(self, *args, **kwargs):
        instance = getattr(self, 'instance')
        delete_old_site = False
        if instance:
            old_instance = TenantSite.objects.filter(id=instance.id).first()
            if old_instance and old_instance.site != self.cleaned_data['site']:
                delete_old_site = True
                old_site = old_instance.site

        instance = super(TenantSiteForm, self).save(*args, **kwargs)

        if delete_old_site:
            old_site.delete()

        return instance


class TenantForm(forms.ModelForm):

    class Meta:
        model = Tenant
        fields = '__all__'

    def clean__extra_data(self):
        extra_data = self.cleaned_data.get('extra_data', {})
        extra_data_helper = TenantExtraDataHelper()
        try:
            validated_extra_data = extra_data_helper.validate_fields(
                {}, extra_data, partial=False)
        except ValidationError as e:
            raise ValidationError(json.dumps(e.message_dict))
        else:
            return validated_extra_data

    def clean__settings(self):
        settings = self.cleaned_data.get('settings', {})
        settings_helper = TenantSettingsHelper()
        try:
            validated_settings = settings_helper.validate_fields(
                {}, settings, partial=False)
        except ValidationError as e:
            raise ValidationError(json.dumps(e.message_dict))
        else:
            return validated_settings


class TenantSiteInLine(admin.StackedInline):
    model = TenantSite
    form = TenantSiteForm
    extra = 0
    min_num = 1


class TenantAdmin(admin.ModelAdmin):
    model = Tenant
    form = TenantForm
    inlines = [TenantSiteInLine]
    prepopulated_fields = {"slug": ("name",)}


admin.site.register(Tenant, TenantAdmin)
admin.site.register(TenantRelationship)



import platform
from django.contrib.sites.shortcuts import get_current_site
from django.utils.functional import SimpleLazyObject
from shared_schema_tenants.settings import TENANT_HTTP_HEADER
from shared_schema_tenants.models import Tenant, TenantSite

if platform.python_version_tuple()[0] == '2':
    import thread as threading
else:
    import threading


def get_tenant(request):
    if not hasattr(request, '_cached_tenant'):
        try:
            request._cached_tenant = get_current_site(request).tenant_site.tenant
            return request._cached_tenant
        except TenantSite.DoesNotExist:
            pass

        try:
            tenant_http_header = 'HTTP_' + TENANT_HTTP_HEADER.replace('-', '_').upper()
            request._cached_tenant = Tenant.objects.get(slug=request.META[tenant_http_header])
        except LookupError:
            lazy_tenant = TenantMiddleware.get_current_tenant()
            if not lazy_tenant:
                return None

            lazy_tenant._setup()
            request._cached_tenant = lazy_tenant._wrapped

    return request._cached_tenant


class TenantMiddleware(object):
    _threadmap = {}

    def __init__(self, get_response):
        self.get_response = get_response
        # One-time configuration and initialization.

    @classmethod
    def get_current_tenant(cls):
        try:
            return cls._threadmap[threading.get_ident()]
        except KeyError:
            return None

    @classmethod
    def set_tenant(cls, tenant_slug):
        cls._threadmap[threading.get_ident()] = SimpleLazyObject(
            lambda: Tenant.objects.get(slug=tenant_slug))

    @classmethod
    def clear_tenant(cls):
        del cls._threadmap[threading.get_ident()]

    def process_request(self, request):
        request.tenant = SimpleLazyObject(lambda: get_tenant(request))
        self._threadmap[threading.get_ident()] = request.tenant

        return request

    def process_exception(self, request, exception):
        try:
            del self._threadmap[threading.get_ident()]
        except KeyError:
            pass

    def process_response(self, request, response):
        try:
            del self._threadmap[threading.get_ident()]
        except KeyError:
            pass
        return response

    def __call__(self, request):
        # Code to be executed for each request before
        # the view (and later middleware) are called.
        request = self.process_request(request)
        response = self.get_response(request)
        return self.process_response(request, response)





from django.db import models
from shared_schema_tenants.settings import DEFAULT_TENANT_SLUG
from shared_schema_tenants.managers import SingleTenantModelManager, MultipleTenantModelManager
from shared_schema_tenants.models import Tenant


def get_default_tenant():
    return Tenant.objects.filter(slug=DEFAULT_TENANT_SLUG).first()


class SingleTenantModelMixin(models.Model):
    tenant = models.ForeignKey('shared_schema_tenants.Tenant', default=get_default_tenant)

    objects = SingleTenantModelManager()

    class Meta:
        abstract = True


class MultipleTenantsModelMixin(models.Model):
    tenants = models.ManyToManyField('shared_schema_tenants.Tenant')

    objects = MultipleTenantModelManager()

    class Meta:
        abstract = True




def import_class(class_path_str):
    last_dot_pos = class_path_str.rfind(".")
    class_name = class_path_str[last_dot_pos + 1:len(class_path_str)]
    class_module = __import__(
        class_path_str[0:last_dot_pos],
        globals(), locals(),
        [class_name]
    )
    return getattr(class_module, class_name)



from django.db import transaction
from django.contrib.sites.models import Site
from django.core.management.base import BaseCommand
from shared_schema_tenants.models import Tenant
from django.utils.text import slugify


class Command(BaseCommand):
    help = 'Creates a new Tenant'

    def handle(self, *args, **options):
        name = input('Enter the Tenant name: ')
        slug = input('Enter the Tenant slug: (%s)' % slugify(name))
        domain = input('Enter the Tenant site: (localhost:8000)')

        if not slug:
            slug = slugify(name)

        if not domain:
            domain = 'localhost:8000'

        with transaction.atomic():
            tenant = Tenant.objects.create(name=name, slug=slug)
            site = Site.objects.create(name=tenant.slug, domain=domain)
            tenant.tenant_sites.create(site=site)

            self.stdout.write(self.style.SUCCESS('Successfully created Tenant %s' % name))



from django.db.models import Manager
from django.core.exceptions import ObjectDoesNotExist
from django.db import transaction
from shared_schema_tenants.helpers.tenants import get_current_tenant
from shared_schema_tenants.exceptions import TenantNotFoundError


class SingleTenantModelManager(Manager):

    def get_queryset(self, tenant=None):
        if not tenant:
            tenant = get_current_tenant()
            if tenant:
                return super(SingleTenantModelManager, self).get_queryset().filter(tenant=tenant)
            else:
                raise TenantNotFoundError()
        else:
            return super(SingleTenantModelManager, self).get_queryset().filter(tenant=tenant)

    def create(self, tenant=None, *args, **kwargs):
        if not tenant:
            tenant = get_current_tenant()
            if tenant:
                kwargs['tenant'] = tenant
                return super(SingleTenantModelManager, self).create(*args, **kwargs)
            else:
                raise TenantNotFoundError()
        else:
            return super(SingleTenantModelManager, self).create(tenant=tenant, *args, **kwargs)


class MultipleTenantModelManager(Manager):

    def _original_get_queryset(self):
        return super(MultipleTenantModelManager, self).get_queryset()

    def get_queryset(self, tenant=None):
        if not tenant:
            tenant = get_current_tenant()
            if tenant:
                return super(MultipleTenantModelManager, self).get_queryset().filter(tenants=tenant)
            else:
                raise TenantNotFoundError()
        else:
            return super(MultipleTenantModelManager, self).get_queryset().filter(tenants=tenant)

    def create(self, tenant=None, *args, **kwargs):
        if not tenant:
            tenant = get_current_tenant()
            if tenant:
                with transaction.atomic():
                    try:
                        model_instance = self._original_get_queryset().get(**kwargs)
                    except ObjectDoesNotExist:
                        model_instance = super(MultipleTenantModelManager, self).create(*args, **kwargs)
                    model_instance.tenants.add(tenant)
                    return model_instance
            else:
                raise TenantNotFoundError()
        else:
            model_instance, created = super(
                MultipleTenantModelManager, self).create(*args, **kwargs)
            model_instance.tenants.add(tenant)
            return model_instance



from shared_schema_tenants.helpers.tenants import get_current_tenant, set_current_tenant
from shared_schema_tenants.helpers.tenant_extra_data import TenantExtraDataHelper
from shared_schema_tenants.helpers.tenant_settings import TenantSettingsHelper
